"""
This module contains StatisticsObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
StatisticsObjects.

    >>> o = r.statisticsobject_add('Test Statistics')

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plutil

from packetlogic2.exceptions import PLDBUnsupportedInFirmware

def _statisticsobjectlist(pldb):
	cid = pldb.session.statistics_object_list()
	return plutil._do(pldb, cid, plutil._ListCB(pldb, pldb.session.statistics_object_list_handle, StatisticsObject))

def _statisticsobjectadd(_pldb, name, total_fields, graph_fields, split, no_root, so_root, graph_freq, min_bytes_in, min_bytes_out):
	kwargs = locals().copy()

	cid = _pldb.session.statistics_object_add(name, total_fields, graph_fields, split, no_root, so_root, graph_freq, min_bytes_in, min_bytes_out)
	(id, tstamp) = plutil._do(_pldb, cid, plutil._AddCB(_pldb, _pldb.session.statistics_object_add_handle))
	res = StatisticsObject(id = id,
			       creation_date = tstamp,
			       modification_date = tstamp,
			       creator = _pldb.user,
			       modified_by = _pldb.user,
			       **kwargs)
	return res

def _statisticsobjectupdate(_pldb, obj, name, total_fields, graph_fields, split, no_root, so_root, graph_freq, min_bytes_in, min_bytes_out):
	cid = _pldb.session.statistics_object_modify(obj.id, name, total_fields, graph_fields, split, no_root, so_root, graph_freq, min_bytes_in, min_bytes_out)
	tstamp = plutil._do(_pldb, cid, plutil._AddCB(_pldb, _pldb.session.statistics_object_modify_handle))
	obj._prop.modification_date = tstamp
	obj._prop.modified_by = _pldb.user

def _rename(pldb, obj, newname):
	cid = pldb.session.statistics_object_rename(obj.id, newname)
	return plutil._do(pldb, cid, plutil._AddCB(pldb, pldb.session.statistics_object_rename_handle))

def _remove(pldb, obj):
	cid = pldb.session.statistics_object_remove(obj.id)
	return plutil._do(pldb, cid, plutil._AddCB(pldb, pldb.session.statistics_object_remove_handle))

_PropatisticsObject = plutil.Prop('id', 'name', 'total_fields', 'graph_fields', 'graph_freq',
				     'min_bytes_in', 'min_bytes_out', 'no_root', 'so_root',
				     'split', 'creator', 'creation_date', 'modified_by',
				     'modification_date')


class StatisticsObject(object):
	"""
	Base class for StatisticsObjects.

	This class should not be instantiated and should instead be created and
	accessed through:

		- L{Ruleset.statisticsobject_add}
		- L{Ruleset.statisticsobject_find}
		- L{Ruleset.statisticsobject_find_id}
		- L{Ruleset.statisticsobject_list}
		- L{Ruleset.statisticsobject_remove}

	Class variables are read-only, use methods to alter the object.

	"""
	__slots__ = ('_prop', '_pldb')

	def __init__(self, _pldb=None, **props):
		self._pldb = _pldb
		self._prop = _PropatisticsObject(**props)

	def __repr__(self):
		return "<PacketLogic StatisticsObject '%s'>" % self._prop.name

	def rename(self, newname):
		"""
		Rename the current object

		I{destroys netobject cache}

		@type newname: C{str}
		@param newname: The name that the object should have after rename process

		"""
		_rename(self._pldb, self, newname)
		self._prop.name = newname

		self._pldb._invalidate_cache()
		
	def _machineid(self):
		""" Use of property 'machineid' is deprecated since v11.0beta5, use 'systemid' instead."""
		import warnings
		warnings.warn("Use of property 'machineid' is deprecated, use 'systemid' instead.",
			      DeprecationWarning, stacklevel=2)
		return self.systemid

	def change(self, total_fields=None, graph_fields=None, graph_freq=None, min_bytes_in=None, min_bytes_out=None, no_root=None, so_root=None, split=None):
		"""
		Change properties of the statistics object. Any unspecified
		(or set to C{None}) property will be left as is.

		@type total_fields: C{int}
		@param total_fields: Bitwise or of one or more of: L{Statistics <statistics.Statistics>}.TOTALFIELD_BYTES_IN, 
		TOTALFIELD_AVG_LATENCY_IN, TOTALFIELD_AVG_LATENCY_OUT, TOTALFIELD_BYTES_IN,
		TOTALFIELD_BYTES_OUT, TOTALFIELD_BYTES_TOTAL, TOTALFIELD_CHANNEL_RX_DROPS,
		TOTALFIELD_CHANNEL_RX_ERRORS, TOTALFIELD_CHANNEL_RX_PACKET, TOTALFIELD_CHANNEL_RX_SPEED,
		TOTALFIELD_CHANNEL_TX_DROPS, TOTALFIELD_CHANNEL_TX_ERRORS, TOTALFIELD_CHANNEL_TX_PACKETS,
		TOTALFIELD_CHANNEL_TX_SPEED, TOTALFIELD_CONNS, TOTALFIELD_CONNS_IN,
		TOTALFIELD_CONNS_OUT, TOTALFIELD_CURCONNS_IN, TOTALFIELD_CURCONNS_OUT,
		TOTALFIELD_PKTDROPS_IN, TOTALFIELD_PKTDROPS_OUT, TOTALFIELD_QUALITY_IN,
		TOTALFIELD_QUALITY_IN_EXT, TOTALFIELD_QUALITY_IN_INT, TOTALFIELD_QUALITY_OUT,
		TOTALFIELD_QUALITY_OUT_EXT, TOTALFIELD_QUALITY_OUT_INT, TOTALFIELD_QUALITY_PKTS_IN,
		TOTALFIELD_QUALITY_PKTS_OUT, TOTALFIELD_RTT_IN, TOTALFIELD_RTT_OUT,
		TOTALFIELD_SUBITEMS, TOTALFIELD_UCONNS, TOTALFIELD_UCONNS_IN,
		TOTALFIELD_UCONNS_OUT

		@type graph_fields: C{int}
		@param graph_fields: Bitwise or of one or more of: L{Statistics <statistics.Statistics>}.GRAPHFIELD_AVG_LATENCY_IN,
		GRAPHFIELD_AVG_LATENCY_OUT, GRAPHFIELD_BPS_IN, GRAPHFIELD_BPS_OUT, GRAPHFIELD_BPS_TOTAL,
		GRAPHFIELD_CHANNEL_RX_DROPS, GRAPHFIELD_CHANNEL_RX_ERRORS, GRAPHFIELD_CHANNEL_RX_PACKETS,
		GRAPHFIELD_CHANNEL_RX_SPEED, GRAPHFIELD_CHANNEL_TX_DROPS, GRAPHFIELD_CHANNEL_TX_ERRORS,
		GRAPHFIELD_CHANNEL_TX_PACKETS, GRAPHFIELD_CHANNEL_TX_SPEED, GRAPHFIELD_CPS, GRAPHFIELD_CPS_IN,
		GRAPHFIELD_CPS_OUT, GRAPHFIELD_CURCONNS_IN, GRAPHFIELD_CURCONNS_OUT, GRAPHFIELD_LINK_UTILIZATION_IN,
		GRAPHFIELD_LINK_UTILIZATION_OUT, GRAPHFIELD_PKTDROPS_IN, GRAPHFIELD_PKTDROPS_OUT, GRAPHFIELD_QUALITY_IN,
		GRAPHFIELD_QUALITY_IN_EXT, GRAPHFIELD_QUALITY_IN_INT, GRAPHFIELD_QUALITY_OUT, GRAPHFIELD_QUALITY_OUT_EXT,
		GRAPHFIELD_QUALITY_OUT_INT, GRAPHFIELD_QUALITY_PKTS_IN, GRAPHFIELD_QUALITY_PKTS_OUT, GRAPHFIELD_RTT_IN,
		GRAPHFIELD_RTT_OUT, GRAPHFIELD_SUBITEMS, GRAPHFIELD_UCPS, GRAPHFIELD_UCPS_IN, GRAPHFIELD_UCPS_OUT,
		GRAPH_MODE_SIZE, GRAPH_MODE_SPEED

		@type min_bytes_in: C{long}
		@param min_bytes_in: Minimum incoming bytes

		@type min_bytes_out: C{long}
		@param min_bytes_out: Minimum outgoing bytes

		@type split: C{*char}
		@param split: Distribution string

		@raises NotImplementedError: If trying to set grap_freq, no_root or so_root
		"""
		if graph_freq is not None or no_root is not None or so_root is not None:
			raise NotImplementedError("Modification of graph_freq, no_root and so_root not implemented yet")
		
		if total_fields is None:
			total_fields = self.total_fields
		if graph_fields is None:
			graph_fields = self.graph_fields
		if min_bytes_in is None:
			min_bytes_in = self.min_bytes_in
		if min_bytes_out is None:
			min_bytes_out = self.min_bytes_out
		if split is None:
			split = self.split

		graph_freq = self.graph_freq
		no_root = self.no_root
		so_root = self.so_root

		_statisticsobjectupdate(self._pldb, self, self._prop.name, total_fields, graph_fields, split, self.no_root, self.so_root, self.graph_freq, min_bytes_in, min_bytes_out)
		
		self._prop.total_fields = total_fields
		self._prop.graph_fields = graph_fields
		self._prop.min_bytes_in = min_bytes_in
		self._prop.min_bytes_out = min_bytes_out
		self._prop.split = split
	
	machineid = property(_machineid)

	id = property(lambda self: self._prop.id, doc="id number of this object")
	name = property(lambda self: self._prop.name, doc="Name of the statistics object")

	total_fields = property(lambda self: self._prop.total_fields, doc="Total fields")
	graph_fields = property(lambda self: self._prop.graph_fields, doc="Graph fields")
	graph_freq = property(lambda self: self._prop.graph_freq, doc="Graph freq")

	min_bytes_in = property(lambda self: self._prop.min_bytes_in, doc="Min bytes in")
	min_bytes_out = property(lambda self: self._prop.min_bytes_out, doc="Min bytes out")

	no_root = property(lambda self: self._prop.no_root, doc="NetObject root")
	so_root = property(lambda self: self._prop.so_root, doc="ServiceObject root")

	split = property(lambda self: self._prop.split, doc="Split string")

	creation_date = property(lambda self: self._prop.creation_date, doc="Date and time when this object was created")
	modification_date = property(lambda self: self._prop.modification_date, doc="Date and time when this object was last modified")
	creator = property(lambda self: self._prop.creator, doc="Username that created this object")
	modified_by = property(lambda self: self._prop.modified_by, doc="Username that last modified this object")

	systemid = plutil._sysidprop("System ID of the PacketLogic this object belongs to. (Replaces machineid)")
	is_global = property(lambda self: False, doc="True if this object resides on another system (always false in 12.1)")
