"""
This module contains PortObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
PortObjects.

    >>> o = r.object_add("/PortObjects/Test")
    >>> o.add("80")
    >>> o.add("6667-6669")
    >>> print o.items
    ['80', '6667-6669']

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plobject
from thisversion import PLDB
import plutil

class PortObject(plobject.PLObject):
	"""
	PortObject Implementation

	Object path: /PortObjects

	@cvar Item: PortObject Item implementation.
	"""

	__slots__ = ('_prop', '_pldb', '_callback', '_item_added')
	_may_exclude = True

	base_path = plutil.roConst("/PortObjects", "Only the type portion of the path")
	type = plutil.roConst(PLDB.OBJECT_TYPE_PORTOBJECT, "Number representing the type of object")

	def _create_item(self, value):
		if isinstance(value, basestring) and "-" in value:
			first, last = value.split("-", 1)
			try:
				item = self.Item(self, value1=first, value2=last, type=PLDB.PORTOBJECT_ITEM_TYPE_RANGE)
			except:
				raise ValueError("Format is not correct for this item type")
				
			if item._prop.value2 < item._prop.value1:
				raise ValueError("First port number should not be smaller than the last port number in a range.")
		else:
			try:
				item = self.Item(self, value1=value, value2=0, type=PLDB.PORTOBJECT_ITEM_TYPE_PORT)
			except:
				raise ValueError("Format is not correct for this item type")

		return item

	def add(self, value, exclude=False):
		"""
		Adds a PortObject item to this PortObject.

		@type value: C{str}
		@param value: The value is a string describing the PortObject item. The type of the item
			will be automatically detected from the string. The following syntax is valid:
				- Single port, just give the plain number I{nn}
				- Port range, give the start and stop port number with a dash as delimiter I{nn-mm}

		@type exclude: C{boolean}
		@param exclude: If this is set to True the item will be a exclude
			Item instead of a include Item.

		"""
		plobject.PLObject.add(self, value, exclude)


	class Item(plobject.PLObject.Item):
		"""
		Portobject Item implementation.
		Should not be instantiated, use L{PortObject.add} instead.

		@type exclude: bool
		@type type: int
		@type value1: int
		@type value2: int
		"""
		__slots__ = ('_prop', '_parent')
		_Prop = plutil.Prop(exclude=bool, type=int, value1=int, value2=int, *plobject._item_props)

		def __str__(self):
			if self._prop.exclude:
				ex = "!"
			else:
				ex = ""

			if self.type == PLDB.PORTOBJECT_ITEM_TYPE_PORT:
				return "%s%d" % (ex, self._prop.value1)
			elif self.type == PLDB.PORTOBJECT_ITEM_TYPE_RANGE:
				return "%s%d-%d" % (ex, self._prop.value1, self._prop.value2)
			else:
				raise ValueError("Wrong item-type for this object")
			
		type = property(lambda self: self._prop.type, doc="Internal number representing the item type (single port, or range)")
		exclude = property(lambda self: self._prop.exclude, doc="True = exclude, False = include")
		value1 = property(lambda self: self._prop.value1, doc="port (or first port in range)")
		value2 = property(lambda self: self._prop.value2, doc="last port in the port range (0 for single port)")
