"""
This module contains VLANObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
VLANObjects.

    >>> o = r.object_add("/VLANObjects/Test")
    >>> o.add("20")
    >>> o.add("1002-1004")
    >>> print o.items
    ['20', '1002-1004']

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plobject
from thisversion import PLDB
import plutil
from plutil import verify_ip

def _verify_vlan(vlan_id):
	if vlan_id < 0 or vlan_id > 4095:
		return False
	return True

class VLANObject(plobject.PLObject):
	"""
	VLANObject Implementation

	Object path: /VLANObjects

	@cvar Item: VLANObject Item implementation.
	"""

	__slots__ = ('_prop', '_pldb', '_callback', '_item_added')
	_may_exclude = True

	base_path = plutil.roConst("/VLANObjects", "Only the type portion of the path")
	type = plutil.roConst(PLDB.OBJECT_TYPE_VLANOBJECT, "Number representing the type of object")

	def _create_item(self, value):
		if isinstance(value, basestring) and "-" in value:
			# type range!
			first, last = value.split("-", 1)
			try:
				item = self.Item(self, type=PLDB.VLANOBJECT_ITEM_TYPE_RANGE, value1=first, value2=last)
			except:
				raise ValueError("Format is not correct for this item type")

			if not _verify_vlan(item._prop.value1) or not _verify_vlan(item._prop.value2):
				raise ValueError("VLAN id must be between 0 and 4095")
			
			if item._prop.value1 > item._prop.value2:
				raise ValueError("Value2 is smaller than Value1")

		else:
			# type address
			item = self.Item(self, type=PLDB.VLANOBJECT_ITEM_TYPE_VLAN, value1=value, value2=0)

			if not _verify_vlan(item._prop.value1):
				raise ValueError("First argument doesn't seem like a VLAN id")

		return item

	def add(self, value, exclude=False):
		"""
		Adds a VLANObject item to this VLANObject.

		@type value: C{str}
		@param value: The value is a string describing the VLANObject item. The type of the item
			will be automatically detected from the string. The following syntax is valid:
				- VLAN ID, just give the plain VLAN id I{XXXX}
				- VLAN ID range, give the start and stop VLAN id with a dash as delimiter I{XXXX-YYYY}

		@type exclude: C{boolean}
		@param exclude: If this is set to True the item will be a exclude
			Item instead of a include Item.

		"""
		plobject.PLObject.add(self, value, exclude)


	class Item(plobject.PLObject.Item):
		"""
		VLANObject Item implementation.
		Should not be instantiated, use L{VLANObject.add} instead.

		@type exclude: C{boolean}
		@type value1: C{int}
		@type value2: C{int}
		"""
		__slots__ = ('_prop', '_parent')
		_Prop = plutil.Prop(exclude=bool, type=int, value1=int, value2=int, *plobject._item_props)

		def __str__(self):
			if self._prop.exclude:
				ex = "!"
			else:
				ex = ""

			if self._prop.type == PLDB.VLANOBJECT_ITEM_TYPE_VLAN:
				return ex + str(self._prop.value1)
			elif self._prop.type == PLDB.VLANOBJECT_ITEM_TYPE_RANGE:
				return ex + "%s-%s" % (self._prop.value1, self._prop.value2)
			else:
				raise ValueError("Wrong item-type for this object")

		type = property(lambda self: self._prop.type, doc="Internal number representing the item type (single or range)")
		exclude = property(lambda self: self._prop.exclude, doc="True = exclude, False = include")
		value1 = property(lambda self: self._prop.value1, doc="VLAN ID, or first ID in a range")
		value2 = property(lambda self: self._prop.value2, doc="0 or last part of the VLAN range")
