"""
This module contains functionality for accessing the plscd part of the
PacketLogic system.

Do NOT import this module directly. The L{PLConnection
<packetlogic2.PLConnection>} object should be used, e.g:

    >>> import packetlogic2
    >>> pl = packetlogic2.connect("192.168.1.25", "admin", "pldemo00")
    >>> plscd = pl.PLSCD()

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import socket
import struct
import threading
import weakref

from packetlogic2.exceptions import PLError


class MsgBuilder(object):
    def __init__(self):
        self.parts = []

    def s16(self, x):
        return self.octets(struct.pack("<h", x))

    def s32(self, x):
        return self.octets(struct.pack("<i", x))

    def s64(self, x):
        return self.octets(struct.pack("<q", x))

    def u64(self, x):
        return self.octets(struct.pack("<Q", x))

    def u32(self, x):
        return self.octets(struct.pack("<I", x))

    def u8(self, x):
        return self.octets(chr(x))

    def s8(self, x):
        return self.octets(chr(x))

    def octets(self, x):
        self.parts.append(x)
        return self

    def inline_blob(self, x):
        return self.u32(len(x)).octets(x)

    def generate(self):
        return "".join(self.parts)


class SchemaHandle(object):
    def __init__(self, plscd, schema):
        self.plscd = weakref.proxy(plscd)
        self.schema = schema

    def create(self, **columnvalues):
        self.plscd._write_msg(PLSCD.SESSION_CONTEXT_CREATE,
                              self.schema.serialize_entry(columnvalues))

    def update(self, **columnvalues):
        self.plscd._write_msg(PLSCD.SESSION_CONTEXT_DELTA,
                              self.schema.serialize_entry(columnvalues))

    def delete(self, **columnvalues):
        self.plscd._write_msg(PLSCD.SESSION_CONTEXT_DELETE,
                              self.schema.serialize_entry(columnvalues))

    def sync_begin(self):
        self.plscd._write_msg(PLSCD.SESSION_CONTEXT_BEGIN, chr(1) + chr(self.schema.id))

    def sync_end(self):
        self.plscd._write_msg(PLSCD.SESSION_CONTEXT_END, "")


def _readloop(p):
    p.conn.settimeout(1.0)
    buf = ""
    while True:
        try:
            if len(buf) < 4:
                r = p.conn.recv(4 - len(buf))
                if len(r) == 0:
                    break
                buf += r
            else:
                cmdlen, = struct.unpack("<I", buf[:4])
                leng = cmdlen & 0xfffff
                if len(buf) < 4 + leng:
                    r = p.conn.recv(4 + leng - len(buf))
                    if len(r) == 0:
                        break
                    buf += r
                if len(buf) == 4 + leng:
                    cmd = cmdlen >> 20
                    p._on_msg(cmd, buf[4:])
                    buf = ""
        except socket.timeout:
            continue


class PLCommdConnector(object):
    def __init__(self, host, username, password):
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.connect((host, 42099))

        service = "plscd"

        d = chr(len(host)) + host + chr(len(service)) + service + chr(0)
        self.sendall(struct.pack("<I", (0x100 << 20) + len(d)) + d)

        cmdlen, = struct.unpack("<I", self.recv(4))
        leng = cmdlen & 0xfffff

        data = ""
        while len(data) < leng:
            data += self.recv(leng - len(data))
        cmd = cmdlen >> 20

        if cmd == 0xfff:  # ABORT
            raise IOError("Error connecting: %r" % data)

        if cmd != 0x101:
            raise IOError("Unknown response: %x" % cmd)

    def close(self):
        self.sock.close()

    def sendall(self, data):
        self.sock.send(data)

    def recv(self, l):
        return self.sock.recv(l)

    def settimeout(self, t):
        self.sock.settimeout(t)


class SSHConnector(object):
    def __init__(self, host, username, password):
        try:
            import paramiko
        except ImportError:
            raise PLError("failed to import paramiko which is needed for plscd")

        self.ssh = paramiko.SSHClient()
        self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.ssh.connect(host, username=username, password=password, port=42007, allow_agent=False, look_for_keys=False)
        self.tpt = self.ssh.get_transport()
        self.chan = self.tpt.open_channel("session")
        self.chan.settimeout(60.0)
        self.chan.invoke_subsystem("packetlogic:plscd")

    def close(self):
        self.chan.close()

    def sendall(self, data):
        self.chan.sendall(data)

    def recv(self, l):
        return self.chan.recv(l)

    def settimeout(self, t):
        self.chan.settimeout(t)


class PLSCD(object):
    """
    Class for connecting and communicating with plscd.

    @raises Realtime.PLError: If the user lacks permission
    for the requested action.

    @group PLSCD: read_msg, write_msg
    """

    OCTETS = 1
    INT16 = 2
    INT32 = 3
    INT64 = 4
    IP_PREFIX = 5
    NOT_IN_KEY = 1
    LOCALIP = 2
    VLAN = 3
    MPLS = 4
    MAC = 5
    PPPOE = 6

    HELLO = 0xff0
    ABORT = 0xfff
    IDENTIFY = 0xff1

    LOOKUP_TABLE = 1
    SESSION_CONTEXT_REGISTER = 2
    SESSION_CONTEXT_BEGIN = 3
    SESSION_CONTEXT_END = 4
    ACCOUNTING_BEGIN = 0x007
    SESSION_CONTEXT_CREATE = 8
    SESSION_CONTEXT_DELTA = 9
    SESSION_CONTEXT_DELETE = 0xa
    ACCOUNTING = 0xb
    ACCOUNTING_ACK = 0xc

    def __init__(self, host, username, password, timeout, connectorclass=SSHConnector):
        self.host = host
        self.username = username
        self.msg_callbacks = {}
        self._schemata = {}

        self.conn = connectorclass(host, username, password)

        self._write_msg(self.IDENTIFY, "PythonAPI client")  # identify as PythonAPI

        # _readloop is outside of the class, and gets the object as a weakref
        # so that the thread doesn't force this object to be alive, but rather terminates
        # when this object is gc:ed.
        self.read_thread = threading.Thread(target=lambda: _readloop(weakref.proxy(self)))
        self.read_thread.daemon = True
        self.read_thread.start()
        # could wait for hello here

    def __repr__(self):
        return '<PLSCD to %s@%s>' % (self.username, self.host)

    def close(self):
        """
        Close connection to PLSCD
        """
        self.conn.close()

    def register(self, schema):
        self._write_msg(self.SESSION_CONTEXT_REGISTER, schema.serialize_register())
        self._schemata[schema.id] = schema
        return SchemaHandle(self, schema)

    def accounting_begin(self, seq, cb=None):
        self._write_msg(self.ACCOUNTING_BEGIN, MsgBuilder().u64(seq).generate())
        if cb is not None:
            def _(data):
                cb(*self._deserialize_counter_message(data))
            self.set_msg_callback(self.ACCOUNTING, _)

    def accounting_ack(self, seq):
        self._write_msg(self.ACCOUNTING_ACK, MsgBuilder().u64(seq).generate())

    def set_msg_callback(self, cmdid, cb):
        self.msg_callbacks[cmdid] = cb

    def _on_msg(self, cmdid, data):
        if cmdid in self.msg_callbacks:
            self.msg_callbacks[cmdid](data)

    def _deserialize_counter_message(self, data):
        seq, = struct.unpack("<Q", data[0:8])
        schemaid = ord(data[8])
        schema = self._schemata[schemaid]
        pos = 9
        fields = {}
        for col in schema.columns:
            if not col.is_sticky:
                continue
            if col.type == self.IP_PREFIX:
                plen = ord(data[pos])
                nbytes = (plen + 7) // 8
                pos += 1
                addr = (data[pos:pos + nbytes] + ("\x00" * 16))
                if plen < 33:
                    val = socket.inet_ntop(socket.AF_INET, addr[:4])
                else:
                    val = socket.inet_ntop(socket.AF_INET6, addr[:16])
                pos += nbytes
            elif col.type == self.INT16:
                val, = struct.unpack("<h", data[pos:pos + 2])
                pos += 2
            elif col.type == self.INT64:
                val, = struct.unpack("<q", data[pos:pos + 8])
                pos += 8
            elif col.type == self.INT32:
                val, = struct.unpack("<i", data[pos:pos + 4])
                pos += 4
            elif col.type == self.OCTETS:
                idorlen, = struct.unpack("<i", data[pos:pos + 4])
                pos += 4
                assert not (idorlen & 0x80000000), "Sorry, stringtable not yet supported"
                val = data[pos:pos + idorlen]
                pos += idorlen
            else:
                assert False, "not supported"

            fields[col.name] = val

        counters = {}
        ncnt = ord(data[pos])
        pos += 1
        for i in range(ncnt):
            cnt_idx = ord(data[pos])
            pos += 1
            inbytes, outbytes = struct.unpack("<II", data[pos:pos + 8])
            pos += 8
            counters[cnt_idx] = (inbytes, outbytes)

        return (seq, schemaid, fields, counters)

    def _write_msg(self, cmd, msg):
        """
        Write message to PLSCD
        @type cmd: C{int}
        @param cmd: One out of
                    SESSION_CONTEXT_REGISTER
                    SESSION_CONTEXT_BEGIN
                    SESSION_CONTEXT_END
                    SESSION_CONTEXT_CREATE
                    SESSION_CONTEXT_DELETE
                    SESSION_CONTEXT_DELTA

        @type msg: C{str}
        @param msg: Message to plscd
        """
        assert 0 <= cmd < 0xfff
        assert len(msg) <= 0xfffff
        cmdlen = struct.pack("<I", (cmd << 20) | len(msg))
        self.conn.sendall(cmdlen + msg)

    class Column(object):
        def __init__(self, name, type, is_sticky, content_type, keytype, keyarg=None):
            self.name = name
            self.type = type
            self.is_sticky = is_sticky
            self.content_type = content_type
            self.keytype = keytype
            self.keyarg = keyarg

    class Counter(object):
        def __init__(self, matches=[], repeat_after_ms=0, repeat_after_bytes=0):
            self.matches = matches
            self.repeat_after_ms = repeat_after_ms
            self.repeat_after_bytes = repeat_after_bytes

    class Schema(object):
        def __init__(self, id, name, timeout, columns, counters=[]):
            self.id = id
            self.name = name
            self.timeout = timeout
            self.columns = columns
            self.counters = counters

        def _find_column(self, colname):
            for i, c in enumerate(self.columns):
                if c.name == colname:
                    return i, c
            raise KeyError("Unknown column %r" % colname)

        def serialize_register(self):
            res = (MsgBuilder()
                   .u8(self.id)
                   .inline_blob(self.name)
                   .u32(self.timeout)
                   .u8(len(self.columns)))
            for f in self.columns:
                (res
                 .inline_blob(f.name)
                 .u8(f.type)
                 .u8(f.is_sticky)
                 .inline_blob(f.content_type)
                 .u8(f.keytype))
                if f.keytype in (PLSCD.VLAN, PLSCD.MPLS):
                    res.s8(f.keyarg)

            # counters
            res.u8(len(self.counters))
            for c in self.counters:
                res.u8(len(c.matches))
                for m in c.matches:
                    res.inline_blob(m)
                res.u32(c.repeat_after_ms)
                res.u32(c.repeat_after_bytes)
            return res.generate()

        def serialize_entry(self, values):
            m = MsgBuilder().u8(self.id).u8(len(values))
            for colname, val in values.iteritems():
                columnidx, columndef = self._find_column(colname)
                m.u8(columnidx)
                if columndef.type == PLSCD.OCTETS:
                    m.inline_blob(val)
                elif columndef.type == PLSCD.INT16:
                    m.s16(val)
                elif columndef.type == PLSCD.INT32:
                    m.s32(val)
                elif columndef.type == PLSCD.INT64:
                    m.s64(val)
                elif columndef.type == PLSCD.IP_PREFIX:
                    ip, plen = val.split("/")
                    plen = int(plen)
                    if ':' in ip:
                        ip = socket.inet_pton(socket.AF_INET6, ip)
                        m.u8(33 + plen)
                        m.octets(ip[:(plen + 7) // 8])
                    else:
                        ip = socket.inet_pton(socket.AF_INET, ip)
                        m.u8(plen)
                        m.octets(ip[:(plen + 7) // 8])
            return m.generate()
