"""
This module contains functionality for managing dynamic liveviews on the
PacketLogic system.

Do NOT import this module directly. The L{PLConnection
<packetlogic2.PLConnection>} object should be used, e.g:

    >>> import packetlogic2
    >>> pl = packetlogic2.connect("192.168.1.25", "admin", "pldemo00")
    >>> dlv = pl.DynamicLiveView()

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

from resource import Resource
from thisversion import PLDB

import plutil

from packetlogic2.exceptions import PLDBUnsupportedInFirmware

_Prop_DLV = plutil.Prop('id', 'name', 'creator', 'items', 'creation_date', 'modification_date', 'modified_by', 'distribution', 'filter')

class DLV(object):

	__slots__ = ('_prop', '_pldb')

	def __init__(self, _pldb=None, **props):
		self._pldb = _pldb
		self._prop = _Prop_DLV(**props)

	def __repr__(self):
		return "<PacketLogic DynamicLiveView '%i' '%s'>" % (self._prop.id, self._prop.name)


	id = property(lambda self: self._prop.id, doc="id number of this dynamicliveview")
	name = property(lambda self: self._prop.name, doc="name of dynamicliveview")
	creator = property(lambda self: self._prop.creator, doc="creator of dynamicliveview")
	creation_date = property(lambda self: self._prop.creation_date, doc="creation date of dynamicliveview")
	modification_date = property(lambda self: self._prop.modification_date, doc="latest modification of dynamicliveview")
	modified_by = property(lambda self: self._prop.modified_by, doc="latest modifier of dynamicliveview")
	distributions = property(lambda self: self._prop.distribution, doc="distributions of this dynamicliveview")
	filters = property(lambda self: self._prop.filter, doc="filters of this dynamicliveview")

	class Distribution(object):
		def __init__(self, distribution, argument=None):
			self.distribution = distribution
			self.argument = argument

		def __repr__(self):
			if self.argument:
				return "<PacketLogic DynamicLiveView Distribution '%s' '%s'>" % (self.distribution, self.argument)
			return "<PacketLogic DynamicLiveView Distribution '%s'>" % (self.distribution)

		"""
		Moves this Distribution up in the order
		"""
		def moveup(self):
			raise NotImplementedError

		"""
		Moves this Distribution down in the order
		"""
		def movedown(self):
			raise NotImplementedError

	class Filter(object):
		def __init__(self, operator, key, values):
			self.operator = operator
			self.key = key
			self.values = values

		def __repr__(self):
			return "<PacketLogic DynamicLiveView filter '%r' '%s' '%s'>" % (self.operator, self.key, self.values)

		"""
		Moves this Filter up in the order
		"""
		def moveup(self):
			raise NotImplementedError

		"""
		Moves this Filter down in the order
		"""
		def movedown(self):
			raise NotImplementedError

	def filter(self, key, *values):
		"""
		Add a filter to the view.

		Makes filter only include connections where the
		property specified by key is one of the specified
		values.

			>>> v.filter("Host", "10.0.0.1")
			>>> v.filter("Server Port", 443)

		@type key: C{str}
		@param key: The key to filter on.

		"""
		operator = 0
		f = DLV.Filter(operator, key, list(values))
		self._prop.filter.append(f)
		cid = self._pldb.session.dynamicliveview_update(self._prop.id, self._prop.filter, self._prop.distribution)
		plutil._do(self._pldb, cid, plutil._AddCB(self._pldb, self._pldb.session.dynamicliveview_update_handle))


	def filter_out(self, key, *values):
		"""
		Add a filter to exclude traffic from the view.

		Makes filter exclude connections where the
		property specified by key is one of the specified
		values.

			>>> v.filter("Host", "10.0.0.1")
			>>> v.filter("Server Port", 443)
			>>> v.filter_out("Service", "SSL v3")

		@type key: C{str}
		@param key: The key to filter on.

		"""
		operator = 1
		f = DLV.Filter(operator, key, list(values))
		self._prop.filter.append(f)
		cid = self._pldb.session.dynamicliveview_update(self._prop.id, self._prop.filter, self._prop.distribution)
		plutil._do(self._pldb, cid, plutil._AddCB(self._pldb, self._pldb.session.dynamicliveview_update_handle))

	def distribution(self, field, arg=""):
		"""
		Add a distribution to the view.

		Adds the specified field as a sub-distribution
		of current distribution.

			>>> v.filter_out("Service", "QUIC")
			>>> v.distribution("Server Hostname")
			>>> v.distribution("NetObject Level", "DHCP/By MAC")
			>>> v.distribution("Local Host")


		@type field: C{str}
		@param field: The field to distribute based on.
		@type arg: C{str}
		@param arg: Argument as required by some fields
		"""
		d = DLV.Distribution(field, arg)
		self._prop.distribution.append(d)
		cid = self._pldb.session.dynamicliveview_update(self._prop.id, self._prop.filter, self._prop.distribution)
		plutil._do(self._pldb, cid, plutil._AddCB(self._pldb, self._pldb.session.dynamicliveview_update_handle))

	def remove(self):
		"""
		Removes this DynamicLiveView from pldb
		"""
		cid = self._pldb.session.dynamicliveview_remove(self._prop.id)
		plutil._do(self._pldb, cid, plutil._Callback(self._pldb))

	def rename(self, name):
		"""
		Renames dynamicliveview to given name
		@type name: C{str}
		@param name: New name
		"""
		cid = self._pldb.session.dynamicliveview_rename(self._prop.id, name)
		plutil._do(self._pldb, cid, plutil._AddCB(self._pldb, self._pldb.session.dynamicliveview_rename_handle))
		self._prop.name = name


class DynamicLiveView(Resource):
	"""
	A connection to a PacketLogic's "DynamicLiveView" resource.

	The methods of this object allows listing, adding,
	removing and modifying dynamic liveviews on the PacketLogic system.

	@group DynamicLiveView: list, add, remove, rename, update
	"""
	def _resource(self):
		return PLDB.RESOURCE_DYNAMICLIVEVIEW

	def list(self):
		"""
		List dynamic liveviews stored on server.

			>>> dlv.list()

		@rtype: C{list} of C{dict}
		@returns: a list of dynamicliveviews

		"""
		return plutil._do(self, self.session.dynamicliveview_list(), plutil._ListCB(self, self.session.dynamicliveview_list_handle, DLV))

	def find(self, name):
		"""
		Find dynamic liveviews stored on server.

			>>> view = dlv.find("view1")

		@rtype: C{dict}
		@returns: DynamicLiveView if found or None if not

		"""
		allviews = self.list()
		for view in allviews:
			if view.name == name:
				return view
		return None

	def add(self, name):
		"""
		Add dynamic liveview with given name to be stored on server.
			>>> dlv.add("HTTP users")
		@rtype: C{int}
		@param name: name of DLV to be created
		@returns: id
		"""
		id = None
		if name:
			cid = self.session.dynamicliveview_add(name)
			id, tstamp = plutil._do(self, cid, plutil._AddCB(self, self.session.dynamicliveview_add_handle))

		return id
